#include <Keypad.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <SD.h> 
#include <TMRpcm.h>
#include <SPI.h>
#include <string.h>

#define SD_ChipSelectPin 10

// player
TMRpcm tmrpcm;

// keypad
const int ROW_NUM = 4;
const int COLUMN_NUM = 4;
char keys[ROW_NUM][COLUMN_NUM] = {
  {'1','2','3','A'},
  {'4','5','6','B'},
  {'7','8','9','C'},
  {'*','0','#','D'}
};
byte pin_rows[ROW_NUM] = {8, 7, 6, 5};
byte pin_column[COLUMN_NUM] = {4, 3, 2, 1};

Keypad keypad = Keypad( makeKeymap(keys), pin_rows, pin_column, ROW_NUM, COLUMN_NUM );

// screen
LiquidCrystal_I2C lcd(0x27, 16, 2);

// create a nice background for when we're playing music
byte image24[8] = {B00100, B00110, B00101, B00101, B00100, B11100, B11100, B11100};
byte image07[8] = {B00111, B00100, B00100, B00100, B00100, B11100, B11100, B11100};
byte image08[8] = {B11100, B00100, B00100, B00100, B00100, B11100, B11100, B11100};
byte image06[8] = {B00100, B00110, B00101, B00101, B00100, B11100, B11100, B11100};
byte image22[8] = {B00111, B00100, B00100, B00100, B00100, B11100, B11100, B11100};
byte image23[8] = {B11100, B00100, B00100, B00100, B00100, B11100, B11100, B11100};
byte image09[8] = {B00100, B00110, B00101, B00101, B00100, B11100, B11100, B11100};
byte image25[8] = {B00000, B01110, B11111, B10101, B11111, B11111, B11111, B10101};

void image() {
  lcd.clear();

  lcd.setCursor(0, 0);
  lcd.write(byte(3));
  lcd.setCursor(0, 1);
  lcd.write(byte(4));
  lcd.setCursor(1, 0);
  lcd.write(byte(1));
  lcd.setCursor(1, 1);
  lcd.write(byte(5));
  lcd.setCursor(2, 0);
  lcd.write(byte(2));
  lcd.setCursor(2, 1);
  lcd.write(byte(0));
  lcd.setCursor(3, 0);
  lcd.write(byte(6));
  lcd.setCursor(3, 1);
  lcd.write(byte(7));

  lcd.setCursor(4, 0);
  lcd.write(byte(3));
  lcd.setCursor(4, 1);
  lcd.write(byte(4));
  lcd.setCursor(5, 0);
  lcd.write(byte(1));
  lcd.setCursor(5, 1);
  lcd.write(byte(5));
  lcd.setCursor(6, 0);
  lcd.write(byte(2));
  lcd.setCursor(6, 1);
  lcd.write(byte(0));
  lcd.setCursor(7, 0);
  lcd.write(byte(6));
  lcd.setCursor(7, 1);
  lcd.write(byte(7));

  lcd.setCursor(8, 0);
  lcd.write(byte(3));
  lcd.setCursor(8, 1);
  lcd.write(byte(4));
  lcd.setCursor(9, 0);
  lcd.write(byte(1));
  lcd.setCursor(9, 1);
  lcd.write(byte(5));
  lcd.setCursor(10, 0);
  lcd.write(byte(2));
  lcd.setCursor(10, 1);
  lcd.write(byte(0));
  lcd.setCursor(11, 0);
  lcd.write(byte(6));
  lcd.setCursor(11, 1);
  lcd.write(byte(7));

  lcd.setCursor(12, 0);
  lcd.write(byte(3));
  lcd.setCursor(12, 1);
  lcd.write(byte(4));
  lcd.setCursor(13, 0);
  lcd.write(byte(1));
  lcd.setCursor(13, 1);
  lcd.write(byte(5));
  lcd.setCursor(14, 0);
  lcd.write(byte(2));
  lcd.setCursor(14, 1);
  lcd.write(byte(0));
  lcd.setCursor(15, 0);
  lcd.write(byte(6));
  lcd.setCursor(15, 1);
  lcd.write(byte(7));
}

void setup()
{
	lcd.begin();
  lcd.backlight();

  tmrpcm.speakerPin = 9;
  // max volume since the speaker is dirt cheap
  tmrpcm.volume('7');
  tmrpcm.loop(true);
  // this does make the sound nicer :D
  tmrpcm.quality(true);

  if (!SD.begin(SD_ChipSelectPin)) {
    // tough luck
    return;
  }

  // display our nice image
  lcd.createChar(0, image24);
  lcd.createChar(1, image07);
  lcd.createChar(2, image08);
  lcd.createChar(3, image06);
  lcd.createChar(4, image22);
  lcd.createChar(5, image23);
  lcd.createChar(6, image09);
  lcd.createChar(7, image25);
  image();
}

char chars[32];
bool displayingBg = true;
int current = 0;
bool paused = false;

void getPrevOrNext(int offset) {
  chars[strlen(chars) - 4] = '\0';
  const int prevOrNext = atoi(chars) + offset;

  sprintf(chars, "%d", prevOrNext);
  current = strlen(chars);
}

void loop()
{
  char key = keypad.getKey();
  if (!key) return;

  if (key == 'A') {                             // pause / resume
    if (!tmrpcm.isPlaying()) return;

    tmrpcm.pause();
  } else if (key == 'B') {                      // previous
    if (tmrpcm.isPlaying()) {
      getPrevOrNext(-1);

      // did a little fancy here
      goto PLAYSONG;
    }
  } else if (key == 'C') {                      // next
    if (tmrpcm.isPlaying()) {
      getPrevOrNext(1);

      // did a little fancy here
      goto PLAYSONG;
    }
  } else if (key == '#') {                      // play chars
PLAYSONG:
    if (current == 0) return;

  	// we stop recording keys and play the song

    // attach file extension
    chars[current++] = '.';
    chars[current++] = 'w';
    chars[current++] = 'a';
    chars[current++] = 'v';
    chars[current++] = '\0';

    // "quality of life"
    lcd.clear();
    lcd.print("playing... ");
    lcd.setCursor(0, 1);
    lcd.print(chars);

    // humans are slow, or so I've heard
    delay(1000);

    tmrpcm.play(chars);

    // display our nice background again
    lcd.clear();
    lcd.setCursor(0, 0);
    image();
    displayingBg = true;

    // reset keys counter
    current = 0;
  } else if (key != 'D' && key != '*') {        // new numkey
    if (displayingBg) {
      lcd.clear();
      displayingBg = false;
      lcd.setCursor(0, 0);
    }
    chars[current++] = key;

    lcd.print(key);
  }
}
